;; window-plus.el --- Provides improved Emacs window commands
;;
;; ver0.01 2015/02/07
;;
;; (reqire 'window-plus)
;; (global-set-key "\C-xo" 'other-window+)
;; (global-set-key "\C-x0" 'delete-window+)
;; (global-set-key "\C-x1" 'delete-other-windows+)
;;

(defcustom other-windowP-ignore-buffers nil
  "other-window+ の対象としないバッファリスト
   設定例
   (setq other-windowP-ignore-buffers (list \"*bijin-tokei*\" \"*Calendar*\"))
  "
  :group 'window-plus)

(defcustom delete-windowP-ignore-buffers nil
  "delete-window+ の対象としないバッファリスト
   設定例
   (setq delete-windowP-ignore-buffers (list \"*bijin-tokei*\" \"*Calendar*\"))
  "
  :group 'window-plus)

(defcustom delete-other-windowsP-ignore-buffers nil
  "delete-other-windows+ の対象としないバッファリスト
   設定例
   (setq delete-other-windowsP-ignore-buffers (list \"*bijin-tokei*\" \"*Calendar*\"))
  "
  :group 'window-plus)


(defun other-window+ (&optional window)
  "other-windowP-ignore-buffers に含まれないバッファウインドウにカレントウインドウを変更する"
  (interactive)

  (let* ((wlist (cdr (window-list)))
         (isleave nil))

    (dolist (win wlist)
      (if (not isleave)
        (let ((ignmatch nil))
          (dolist (bufname other-windowP-ignore-buffers)
            (if (not (eq (get-buffer-window bufname) nil))
                (dolist (ignwin (get-buffer-window-list bufname))
                  (if (eq win ignwin) (setq ignmatch t)))))

          (if (not ignmatch)
              (progn
                (select-window win)
                (setq isleave t))))))
    )
  )

(defun delete-window+ (&optional window)
  "delete-windowP-ignore-buffers に含まれるバッファウインドウを残したままカレントウインドウを閉じる"
  (interactive)

  (let* ((wlist (window-list))
         (considerwlist))

    (dolist (win wlist)
      (let ((ignmatch nil))
        (dolist (bufname delete-windowP-ignore-buffers)
          (if (not (eq (get-buffer-window bufname) nil))
              (dolist (ignwin (get-buffer-window-list bufname))
                (if (eq win ignwin) (setq ignmatch t)))))

        (if (not ignmatch)
            (setq considerwlist (append considerwlist (list win))))))
    (if (not (eq (length considerwlist) 0))
        (if (not (eq (length considerwlist) 1))
            (progn
              (delete-window (car considerwlist))
              (select-window (car (cdr considerwlist))))))
    )
  )

(defun delete-other-windows+ (&optional window)
  "カレントウインドウと delete-other-windowsP-ignore-buffers に含まれないバッファウインドウを閉じる"
  (interactive)

  (let* ((wlist (window-list))
         (curwin (car (get-buffer-window-list)))
         (delwins))

    (dolist (win wlist)
      (if (not (eq win curwin))
        (let ((ignmatch nil))
          (dolist (bufname delete-other-windowsP-ignore-buffers)
            (if (not (eq (get-buffer-window bufname) nil))
                (dolist (ignwin (get-buffer-window-list bufname))
                  (if (eq win ignwin) (setq ignmatch t)))))
          (if (not ignmatch) (push win delwins)))))
    (dolist (win delwins)
      (delete-window win))
    )
  )

(provide 'window-plus)
